package com.qgs.dc.s7.retrydemo.forTest;

import com.qgs.dc.s7.my.s7connector.api.S7Connector;
import com.qgs.dc.s7.my.s7connector.enmuc.PlcVarActual;
import com.qgs.dc.s7.my.s7connector.enmuc.S7Client;
import com.qgs.dc.s7.my.s7connector.enmuc.S7ClientNew;
import com.qgs.dc.s7.my.s7connector.exception.S7ParseDataException;
import com.qgs.dc.s7.my.s7connector.type.PlcVar;
import org.apache.commons.lang3.RandomStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

/**
 * @Desc: ""
 * @Author: caixiang
 * @DATE: 2022/12/23 12:33
 */
@Component
@Order(value = 1)
public class InitialS7Thread implements ApplicationRunner {
    private static final Logger logger = LoggerFactory.getLogger(InitialS7Thread.class);

    public InitialS7Thread(){
        this.executor = Executors.newScheduledThreadPool(10);
    }

    private ScheduledExecutorService executor;

    private void write(S7ClientNew s7Client,PlcVarActual var,Object newValue) {
        if(var.getType().equals(PlcVar.STRING_Array)){
            String[] s = (String[])newValue;
            String[] ss = (String[])newValue;
            if(s.length > var.getLength() ){
                ss = new String[var.getLength()];
                for(int i=0;i< var.getLength();i++){
                    ss[i] = s[i];
                }
            }
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),ss);
        }else {
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),newValue);
        }
    }
    private Object read(S7ClientNew s7Client,PlcVarActual var) {
        try {
            return s7Client.read(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getLength(), var.getStrSize(), var.getType());
        }catch (Exception e){
            logger.error("host:"+s7Client.getHost()+" ; read 操作出现问题: "+e.getMessage());
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {

        executor.execute(new Runnable() {
            @Override
            public void run() {
                while (true){
                    //read one
                    logger.info(Thread.currentThread().getName()+" , subId : "+Arrays.toString((String[])read(S7ClientNew.S7_1500,PlcVarActual.SubIdArrays)));
                    try {
                        Thread.sleep(300);
                    } catch (InterruptedException e) {
                        throw new S7ParseDataException(e);
                    }


                    //write one
                    String[] toWrite = new String[60];
                    for(int i=0;i<60;i++){
                        toWrite[i] = RandomStringUtils.randomAlphanumeric(18);

                    }
                    try {
                        write(S7ClientNew.S7_1500,PlcVarActual.SubIdArrays,toWrite);
                    } catch (Exception e) {
                        throw new S7ParseDataException(e);
                    }

                }
            }
        });
        executor.execute(new Runnable() {
            @Override
            public void run() {
                while (true){
                    logger.info(Thread.currentThread().getName()+" , subId : "+Arrays.toString((String[])read(S7ClientNew.S7_1500,PlcVarActual.SubIdArrays)));

                    try {
                        Thread.sleep(300);
                    } catch (InterruptedException e) {
                        throw new RuntimeException(e);
                    }

                    //write one
                    String[] toWrite = new String[60];
                    for(int i=0;i<60;i++){
                        toWrite[i] = RandomStringUtils.randomAlphanumeric(18);

                    }
                    try {
                        write(S7ClientNew.S7_15001,PlcVarActual.SubIdArrays,toWrite);
                    } catch (Exception e) {
                        throw new S7ParseDataException(e);
                    }
                }
            }
        });


    }
}
